@@ -33,7 +33,6 @@
 
 #include "xwayland-types.h"
 
-#ifdef GLAMOR_HAS_GBM
 struct xwl_present_window {
     WindowPtr window;
 
@@ -51,15 +50,27 @@ struct xwl_present_window {
     struct xorg_list wait_list;
     struct xorg_list flip_queue;
     struct xorg_list idle_queue;
+    struct xorg_list blocked_queue;
 
     present_vblank_ptr flip_active;
+    uint64_t blocking_event;
+
+    OsTimerPtr unredirect_timer;
+    Bool redirected;
+    Bool redirect_failed;
 };
 
 struct xwl_present_event {
     present_vblank_rec vblank;
 
     PixmapPtr pixmap;
-    Bool async_may_tear;
+    Bool copy_executed;
+
+    uint32_t options;
+    uint64_t divisor;
+    uint64_t remainder;
+
+    struct xorg_list blocked;
 };
 
 Bool xwl_present_entered_for_each_frame_callback(void);
@@ -70,7 +81,7 @@ void xwl_present_frame_callback(struct xwl_present_window *xwl_present_window);
 Bool xwl_present_init(ScreenPtr screen);
 void xwl_present_cleanup(WindowPtr window);
 void xwl_present_unrealize_window(struct xwl_present_window *xwl_present_window);
-
-#endif /* GLAMOR_HAS_GBM */
+Bool xwl_present_maybe_redirect_window(WindowPtr window, PixmapPtr pixmap);
+Bool xwl_present_maybe_unredirect_window(WindowPtr window);
 
 #endif /* XWAYLAND_PRESENT_H */
