@@ -61,60 +61,76 @@ SOFTWARE.
 #define LATER 1
 
 #define NullClient ((ClientPtr) 0)
-#define REQUEST(type) \
-	type *stuff = (type *)client->requestBuffer
+
+#define REQUEST(type)                                                   \
+    type * stuff = (type *)client->requestBuffer;
 
 #define ARRAY_SIZE(a)  (sizeof((a)) / sizeof((a)[0]))
 
-#define REQUEST_SIZE_MATCH(req)\
-    if ((sizeof(req) >> 2) != client->req_len)\
-         return(BadLength)
+#define REQUEST_SIZE_MATCH(req)                                         \
+    do {                                                                \
+        if ((sizeof(req) >> 2) != client->req_len)                      \
+            return(BadLength);                                          \
+    } while (0)
 
-#define REQUEST_AT_LEAST_SIZE(req) \
-    if ((sizeof(req) >> 2) > client->req_len )\
-         return(BadLength)
+#define REQUEST_AT_LEAST_SIZE(req)                                      \
+    do {                                                                \
+        if ((sizeof(req) >> 2) > client->req_len)                       \
+            return(BadLength);                                          \
+    } while (0)
 
-#define REQUEST_AT_LEAST_EXTRA_SIZE(req, extra)  \
-    if (((sizeof(req) + ((uint64_t) extra)) >> 2) > client->req_len ) \
-         return(BadLength)
+#define REQUEST_AT_LEAST_EXTRA_SIZE(req, extra)                         \
+    do {                                                                \
+        if (((sizeof(req) + ((uint64_t) (extra))) >> 2) > client->req_len) \
+            return(BadLength);                                          \
+    } while (0)
 
-#define REQUEST_FIXED_SIZE(req, n)\
-    if (((sizeof(req) >> 2) > client->req_len) || \
-        (((n) >> 2) >= client->req_len) ||                              \
-        ((((uint64_t) sizeof(req) + (n) + 3) >> 2) != (uint64_t) client->req_len))  \
-         return(BadLength)
+#define REQUEST_FIXED_SIZE(req, n)                                      \
+    do {                                                                \
+        if ((((sizeof(req)) >> 2) > client->req_len) ||            \
+            (((n) >> 2) >= client->req_len) ||                         \
+            ((((uint64_t) sizeof(req) + (n) + 3) >> 2) != (uint64_t) client->req_len)) \
+            return(BadLength);                                          \
+    } while (0)
 
-#define LEGAL_NEW_RESOURCE(id,client)\
-    if (!LegalNewID(id,client)) \
-    {\
-	client->errorValue = id;\
-        return BadIDChoice;\
-    }
+#define LEGAL_NEW_RESOURCE(id,client)           \
+    do {                                        \
+        if (!LegalNewID((id), (client))) {      \
+            (client)->errorValue = (id);        \
+            return BadIDChoice;                 \
+        }                                       \
+    } while (0)
 
-#define VALIDATE_DRAWABLE_AND_GC(drawID, pDraw, mode)\
-    {\
-	int tmprc = dixLookupDrawable(&(pDraw), drawID, client, M_ANY, mode);\
-	if (tmprc != Success)\
-	    return tmprc;\
-	tmprc = dixLookupGC(&(pGC), stuff->gc, client, DixUseAccess);\
-	if (tmprc != Success)\
-	    return tmprc;\
-	if ((pGC->depth != pDraw->depth) || (pGC->pScreen != pDraw->pScreen))\
-	    return BadMatch;\
-    }\
-    if (pGC->serialNumber != pDraw->serialNumber)\
-	ValidateGC(pDraw, pGC);
+#define VALIDATE_DRAWABLE_AND_GC(drawID, pDraw, mode)                   \
+    do {                                                                \
+        int tmprc = dixLookupDrawable(&(pDraw), drawID, client, M_ANY, mode); \
+        if (tmprc != Success)                                           \
+            return tmprc;                                               \
+        tmprc = dixLookupGC(&(pGC), stuff->gc, client, DixUseAccess);   \
+        if (tmprc != Success)                                           \
+            return tmprc;                                               \
+        if ((pGC->depth != pDraw->depth) || (pGC->pScreen != pDraw->pScreen)) \
+            return BadMatch;                                            \
+        if (pGC->serialNumber != pDraw->serialNumber)                   \
+            ValidateGC(pDraw, pGC);                                     \
+    } while (0)
 
-#define WriteReplyToClient(pClient, size, pReply) { \
-   if ((pClient)->swapped) \
-      (*ReplySwapVector[((xReq *)(pClient)->requestBuffer)->reqType]) \
-           (pClient, (int)(size), pReply); \
-   else WriteToClient(pClient, (int)(size), (pReply)); }
+#define WriteReplyToClient(pClient, size, pReply)                       \
+    do {                                                                \
+        if ((pClient)->swapped)                                         \
+            (*ReplySwapVector[((xReq *)(pClient)->requestBuffer)->reqType]) \
+                (pClient, (int)(size), pReply);                         \
+        else                                                            \
+            WriteToClient(pClient, (int)(size), (pReply));              \
+    } while (0)
 
-#define WriteSwappedDataToClient(pClient, size, pbuf) \
-   if ((pClient)->swapped) \
-      (*(pClient)->pSwapReplyFunc)(pClient, (int)(size), pbuf); \
-   else WriteToClient(pClient, (int)(size), (pbuf));
+#define WriteSwappedDataToClient(pClient, size, pbuf)                   \
+    do {                                                                \
+        if ((pClient)->swapped)                                         \
+            (*(pClient)->pSwapReplyFunc)(pClient, (int)(size), pbuf);   \
+        else                                                            \
+            WriteToClient(pClient, (int)(size), (pbuf));                \
+    } while (0)
 
 typedef struct _TimeStamp *TimeStampPtr;
 
@@ -129,32 +145,21 @@ typedef struct _WorkQueue *WorkQueuePtr;
 extern _X_EXPORT ClientPtr clients[MAXCLIENTS];
 extern _X_EXPORT ClientPtr serverClient;
 extern _X_EXPORT int currentMaxClients;
-extern _X_EXPORT char dispatchExceptionAtReset;
-
-typedef int HWEventQueueType;
-typedef HWEventQueueType *HWEventQueuePtr;
-
-extern _X_EXPORT HWEventQueuePtr checkForInput[2];
-
-static inline _X_NOTSAN Bool
-InputCheckPending(void)
-{
-    return (*checkForInput[0] != *checkForInput[1]);
-}
 
 typedef struct _TimeStamp {
     CARD32 months;              /* really ~49.7 days */
     CARD32 milliseconds;
 } TimeStamp;
 
+typedef int HWEventQueueType;
+typedef HWEventQueueType *HWEventQueuePtr;
+
 /* dispatch.c */
 extern _X_EXPORT ClientPtr GetCurrentClient(void);
 
 extern _X_EXPORT void SetInputCheck(HWEventQueuePtr /*c0 */ ,
                                     HWEventQueuePtr /*c1 */ );
 
-extern _X_EXPORT void CloseDownClient(ClientPtr /*client */ );
-
 extern _X_EXPORT void UpdateCurrentTime(void);
 
 extern _X_EXPORT void UpdateCurrentTimeIf(void);
@@ -241,12 +246,6 @@ extern _X_EXPORT void RemoveBlockAndWakeupHandlers(ServerBlockHandlerProcPtr blo
 
 extern _X_EXPORT void InitBlockAndWakeupHandlers(void);
 
-extern _X_EXPORT void ClearWorkQueue(void);
-
-extern _X_EXPORT void ProcessWorkQueue(void);
-
-extern _X_EXPORT void ProcessWorkQueueZombies(void);
-
 extern _X_EXPORT Bool QueueWorkProc(Bool (*function)(ClientPtr clientUnused,
                                                      void *closure),
                                     ClientPtr client,
@@ -421,6 +420,10 @@ DeliverTouchEvents(DeviceIntPtr /* dev */ ,
                    InternalEvent * /* ev */ ,
                    XID /* resource */ );
 
+extern Bool
+DeliverGestureEventToOwner(DeviceIntPtr dev, GestureInfoPtr gi,
+                           InternalEvent *ev);
+
 extern void
 InitializeSprite(DeviceIntPtr /* pDev */ ,
                  WindowPtr /* pWin */ );
@@ -436,7 +439,7 @@ WindowHasNewCursor(WindowPtr /* pWin */ );
 
 extern Bool
 CheckDeviceGrabs(DeviceIntPtr /* device */ ,
-                 DeviceEvent * /* event */ ,
+                 InternalEvent * /* event */ ,
                  WindowPtr /* ancestor */ );
 
 extern void
@@ -548,11 +551,6 @@ IsInterferingGrab(ClientPtr /* client */ ,
                   DeviceIntPtr /* dev */ ,
                   xEvent * /* events */ );
 
-#ifdef PANORAMIX
-extern _X_EXPORT void
-ReinitializeRootWindow(WindowPtr win, int xoff, int yoff);
-#endif
-
 #ifdef RANDR
 extern _X_EXPORT void
 ScreenRestructured(ScreenPtr pScreen);
@@ -610,6 +608,13 @@ extern Bool
 IsPointerEvent(InternalEvent *event);
 extern Bool
 IsTouchEvent(InternalEvent *event);
+Bool
+IsGestureEvent(InternalEvent *event);
+Bool
+IsGestureBeginEvent(InternalEvent *event);
+Bool
+IsGestureEndEvent(InternalEvent *event);
+
 extern _X_EXPORT Bool
 IsMaster(DeviceIntPtr dev);
 extern _X_EXPORT Bool
