@@ -21,7 +21,7 @@
  * OF THIS SOFTWARE.
  */
 
-#include "randrstr.h"
+#include "randrstr_priv.h"
 #include "swaprep.h"
 #include "mipointer.h"
 
