@@ -23,7 +23,7 @@
  * Authors: Dave Airlie
  */
 
-#include "randrstr.h"
+#include "randrstr_priv.h"
 #include "swaprep.h"
 
 #include <X11/Xatom.h>
@@ -459,7 +459,7 @@ RRProviderInit(void)
     return TRUE;
 }
 
-extern _X_EXPORT Bool
+Bool
 RRProviderLookup(XID id, RRProviderPtr *provider_p)
 {
     int rc = dixLookupResourceByType((void **)provider_p, id,
